/*
By John Haggerty
Animated Mobile
*/

#include <GL/glut.h>
#include <cstdlib>
#include <ctime>
#include <cmath>

#include <iostream>
#include <vector>

#include "../../png_reader/pngread.h"

using namespace std;

const double PI = 4*atan(1);

//=============== VARIOUS FUNCTIONS =========================

double starttime, lastanimtime, frametime;
double elapsedTime()
{
	return (clock() - starttime)/CLOCKS_PER_SEC;
}

GLdouble min(const GLdouble a, const GLdouble b)
{
	return (a<b)?a:b;
}

template <class T>
void removeVectorElement(vector<T> &vec, const int &i)
{
	for (int a=i; a < vec.size()-1; a++)
	{
		vec[a] = vec[a+1];
	}
	vec.pop_back();
}

//=============== VARS =========================

const float sunambient[3] = {0.1, 0.09, 0.08};
const float sundiffuse[3] = {0.9, 0.75, 0.7};
const float sunspecular[3] = {1.0, 0.9, 0.85};
const float sunpos[4] = {-0.5, 0.7, 1.0, 0.0}; // from position of sun on background image

const float groundlightambient[3] = {0.03, 0.04, 0.05};
const float groundlightdiffuse[3] = {0.2, 0.22, 0.4};
const float groundlightspecular[3] = {0.0, 0.0, 0.0};
const float groundlightpos[4] = {0.0, -1.0, 0.0, 0.0}; // from below

const float skylightambient[3] = {0.0, 0.0, 0.0};
const float skylightdiffuse[3] = {0.4, 0.2, 0.3};
const float skylightspecular[3] = {0.0, 0.0, 0.0};
const float skylightpos[4] = {0.4, 0.6, -1.0, 0.0}; // from the opposite side of the sky as the sun

const float fullcolor[3] = {1.0, 1.0, 1.0};
const float   nocolor[3] = {0.0, 0.0, 0.0};

const float stringcolor[3] = {0.6, 0.4, 0.3};

// a Bar struct, represents a bar in the mobile with two other bars hanging off of it.
// this is essentially a binary tree.
struct Bar {
	// is this really a bar, or is it just a hanging object (leaf node)?
	bool isObject; 

	union {
		// data for bars
		struct {
			// the left and right bars hanging from this one
			Bar *left;
			Bar *right;

			// how far away from the center of this bar the two sub-bars are hanging
			double leftDistance, rightDistance;
		} barData;

		// data for objects
		struct {
			double weight;
			GLuint displaylist;
		} objectData;
	} data;

	// the length of the string that this bar/object is hanging from
	double stringLength;

	// current rotation, current rotational velocity,
	// and a rotational acceleration factor representing
	// how much the string which this is hanging from is willing to twist.
	double rot, rotvel, rotaccel;
};

Bar mobile; // the top-level bar.

bool mobilegenerated;

const double barheight = 0.1;
const double barthickness = 0.2;
const double barweightperunit = 0.02;
const double mobileweight = 2.0;

// textures
GLuint bgtextures[6], bartexture;
double skytextureoffset;

// nurbs stuff
GLUnurbsObj *myNurb;

bool mousedragging = false;
double mx, my; // mouse positions

double mobileHeight;
double viewDist;
double viewTheta = PI / 4.0;
double viewPhi = PI / 16.0;

GLdouble xoffset, yoffset, xscale, yscale; // used for mouse coordinate conversion

//=============== ANIMATION FUNCTION =========================

void animateBar(Bar &bar, const double &frametime)
{
	if (!bar.isObject)
	{
		// animate the sub-bars
		if (bar.data.barData.left != NULL)
			animateBar(*bar.data.barData.left, frametime);
		if (bar.data.barData.right != NULL)
			animateBar(*bar.data.barData.right, frametime);
	}

	// animate rotation
	bar.rot += bar.rotvel * frametime;
	bar.rotvel -= bar.rotaccel * bar.rot * frametime;
}

// (used for glut idle loop, animates the mobile)
void animate()
{
	frametime = elapsedTime() - lastanimtime;
	if (frametime == 0) return;

	//if (!mousedragging) // don't animate while the user is moving the view
	{
		// recurse through the tree and animate each bar
		animateBar(mobile, frametime);
	}

	lastanimtime += frametime;
	glutPostRedisplay(); // update display
}

//=============== DISPLAY FUNCTIONS =========================

void displayBackground(const double campos[3])
{
	int a;

	// display a textured cube centered on the camera for the background.
	
	// ignore the depth buffer while drawing the background, which should appear
	// to be infinitely far away.
	glDisable(GL_DEPTH_TEST);

	glEnable(GL_TEXTURE_2D);

	glColor3d(1.0,1.0,1.0);

	glPushMatrix();
	glTranslated(campos[0],campos[1],campos[2]);

	// draw the six faces of the cube
	for (a=0; a < 6; a++)
	{
		glBindTexture(GL_TEXTURE_2D, bgtextures[a]);

		glPushMatrix();

		// choose the orientation of this polygon based on which face we're drawing
		switch (a)
		{
			case 0:
				glRotated( 90.0, 0.0,1.0,0.0);
				break;
			case 1:
				glRotated(-90.0, 0.0,1.0,0.0);
				break;
			case 2:
				glRotated(-90.0, 1.0,0.0,0.0);
				break;
			case 3:
				glRotated( 90.0, 1.0,0.0,0.0);
				break;
			case 4:
				// no rotation
				break;
			case 5:
				glRotated(180.0, 0.0,1.0,0.0);
				break;
		}

		glBegin(GL_QUADS);
			// taking a slight subarea of the texture in order to not require opengl 1.2 for GL_CLAMP_EDGE.
			// otherwise, we get a slight black border around the edge of the texture, due to "texture borders".
			glTexCoord2d(0.0 + skytextureoffset, 0.0 + skytextureoffset); glVertex3d(-1.0, -1.0, 1.0);
			glTexCoord2d(1.0 - skytextureoffset, 0.0 + skytextureoffset); glVertex3d( 1.0, -1.0, 1.0);
			glTexCoord2d(1.0 - skytextureoffset, 1.0 - skytextureoffset); glVertex3d( 1.0,  1.0, 1.0);
			glTexCoord2d(0.0 + skytextureoffset, 1.0 - skytextureoffset); glVertex3d(-1.0,  1.0, 1.0);
		glEnd();

		glPopMatrix();
	}

	glPopMatrix();

	glDisable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
}

void displayBar(const Bar &bar)
{
	double scaleby;
	// rotate this bar/object
	glRotated(bar.rot / PI * 180.0, 0.0,1.0,0.0);

	glColor3fv(stringcolor);

	// make the string from the origin down to y = -stringLength
	glBegin(GL_LINES);
		glVertex3d(0.0,0.0,0.0);
		glVertex3d(0.0,-bar.stringLength,0.0);
	glEnd();

	// translate the rest of what's drawn down to the bottom of the string
	glTranslated(0.0, -bar.stringLength, 0.0);

	if (bar.isObject)
	{
		glEnable(GL_LIGHTING);
		glEnable(GL_COLOR_MATERIAL);

		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fullcolor);
		glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 20.0);
		
		glPushMatrix();
			scaleby = pow(bar.data.objectData.weight, 1.0/3.0) * 0.3;
			glScaled(scaleby,scaleby,scaleby);

			glCallList(bar.data.objectData.displaylist);

		glPopMatrix();

		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, nocolor);

		glDisable(GL_COLOR_MATERIAL);
		glDisable(GL_LIGHTING);
	}
	else
	{
		// display the actual bar

		glBindTexture(GL_TEXTURE_2D, bartexture);
		glEnable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);

		glPushMatrix();
			glTranslated(bar.data.barData.leftDistance - barheight, -barheight/2.0, -barthickness/2.0);
			glScaled(bar.data.barData.rightDistance - bar.data.barData.leftDistance + 2.0*barheight, barheight, barthickness);

			glBegin(GL_QUADS);
				// bottom
				glNormal3d( 0.0,-1.0, 0.0);
				glTexCoord2d( 0.0, 0.0); glVertex3d(0.0, 0.0, 0.0);
				glTexCoord2d( 1.0, 0.0); glVertex3d(1.0, 0.0, 0.0);
				glTexCoord2d( 1.0, 0.3); glVertex3d(1.0, 0.0, 1.0);
				glTexCoord2d( 0.0, 0.3); glVertex3d(0.0, 0.0, 1.0);

				// top
				glNormal3d( 0.0, 1.0, 0.0);
				glTexCoord2d( 0.0, 0.7); glVertex3d(0.0, 1.0, 0.0);
				glTexCoord2d( 1.0, 0.7); glVertex3d(1.0, 1.0, 0.0);
				glTexCoord2d( 1.0, 0.4); glVertex3d(1.0, 1.0, 1.0);
				glTexCoord2d( 0.0, 0.4); glVertex3d(0.0, 1.0, 1.0);

				// front
				glNormal3d( 0.0, 0.0,-1.0);
				glTexCoord2d( 0.0, 0.8); glVertex3d(0.0, 0.0, 0.0);
				glTexCoord2d( 1.0, 0.8); glVertex3d(1.0, 0.0, 0.0);
				glTexCoord2d( 1.0, 0.7); glVertex3d(1.0, 1.0, 0.0);
				glTexCoord2d( 0.0, 0.7); glVertex3d(0.0, 1.0, 0.0);

				// back
				glNormal3d( 0.0, 0.0, 1.0);
				glTexCoord2d( 0.0, 0.3); glVertex3d(0.0, 0.0, 1.0);
				glTexCoord2d( 1.0, 0.3); glVertex3d(1.0, 0.0, 1.0);
				glTexCoord2d( 1.0, 0.4); glVertex3d(1.0, 1.0, 1.0);
				glTexCoord2d( 0.0, 0.4); glVertex3d(0.0, 1.0, 1.0);

				// left
				glNormal3d(-1.0, 0.0, 0.0);
				glTexCoord2d( 0.0, 0.0); glVertex3d(0.0, 0.0, 0.0);
				glTexCoord2d( 1.0, 0.0); glVertex3d(0.0, 0.0, 1.0);
				glTexCoord2d( 1.0, 1.0); glVertex3d(0.0, 1.0, 1.0);
				glTexCoord2d( 0.0, 1.0); glVertex3d(0.0, 1.0, 0.0);

				// right
				glNormal3d( 1.0, 0.0, 0.0);
				glTexCoord2d( 0.0, 0.0); glVertex3d(1.0, 0.0, 0.0);
				glTexCoord2d( 1.0, 0.0); glVertex3d(1.0, 0.0, 1.0);
				glTexCoord2d( 1.0, 1.0); glVertex3d(1.0, 1.0, 1.0);
				glTexCoord2d( 0.0, 1.0); glVertex3d(1.0, 1.0, 0.0);
			glEnd();
		glPopMatrix();

		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);

		// display the sub-bars

		// save the current matrix state so that we can move back up the heirarchy later
		glPushMatrix();

			// transform the sub-bar to where we want it to be
			glTranslated(bar.data.barData.leftDistance, 0.0, 0.0);
			
			// draw this sub-bar
			displayBar(*bar.data.barData.left);

		// return back to the current bar's level of the heirarchy
		glPopMatrix();

		// save the current matrix state so that we can move back up the heirarchy later
		glPushMatrix();

			// transform the sub-bar to where we want it to be
			glTranslated(bar.data.barData.rightDistance, 0.0, 0.0);
			
			// draw this sub-bar
			displayBar(*bar.data.barData.right);

		// return back to the current bar's level of the heirarchy
		glPopMatrix();
	}
}

void display(void)
{
	double campos[3];

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

	// camera
	campos[0] = viewDist * cos(viewTheta) * cos(viewPhi);
	campos[1] = viewDist * sin(viewPhi) - mobileHeight / 2.0;
	campos[2] = viewDist * sin(viewTheta) * cos(viewPhi);

	gluLookAt(
		campos[0], campos[1], campos[2],
		0.0, -mobileHeight / 2.0, 0.0,
		0.0, (((int)floor(((viewPhi - PI/2.0)/PI)) % 2) ? 1.0 : -1.0), 0.0 // choose sky direction from +y or -y depending on whether viewPhi indicates that the view is upsidedown or not
	);

	// background
	displayBackground(campos);

	glLightfv(GL_LIGHT0, GL_POSITION, sunpos);
	glLightfv(GL_LIGHT1, GL_POSITION, groundlightpos);
	glLightfv(GL_LIGHT2, GL_POSITION, skylightpos);

	if (mobilegenerated)
	{
		// make a string which from the origin upwards which fades to transparent
		glEnable(GL_BLEND);
		glBegin(GL_LINES);
			glColor3fv(stringcolor);
			glVertex3d(0.0,0.0,0.0);
			glColor4f(stringcolor[0], stringcolor[1], stringcolor[2], 0.0);
			glVertex3d(0.0,3.0,0.0);
		glEnd();
		glDisable(GL_BLEND);

		glPushMatrix();
			// display the mobile recursively
			displayBar(mobile);
		glPopMatrix();
	}

	glutSwapBuffers();
	glFlush();
}

//=============== INIT =========================

void renderRandomNurbsObject()
{
	int a, x,y,z, sampx,sampy,sampz, xcoord,ycoord,zcoord, xmult,ymult,zmult;
	GLfloat *knots;
	GLfloat *ctrlpoints;
	GLfloat *randvecs;

	// create a distorted cube hanging from the origin made up of four bezier surfaces

	// (rotate randomly)
	glRotated(((double)rand()/RAND_MAX) * 360.0, 1.0,0.0,0.0);
	glRotated(((double)rand()/RAND_MAX) * 360.0, 0.0,1.0,0.0);
	glRotated(((double)rand()/RAND_MAX) * 360.0, 0.0,0.0,1.0);

	// 8 knots used for bezier surfaces: 0,0,0,0,1,1,1,1
	knots = new GLfloat[8];
	knots[0]=0.0;knots[1]=0.0;knots[2]=0.0;knots[3]=0.0;
	knots[4]=1.0;knots[5]=1.0;knots[6]=1.0;knots[7]=1.0;

	ctrlpoints = new GLfloat[4*4*3];
	randvecs = new GLfloat[4*4*4*3]; // 4x4x4 random vectors (middle 2x2 will never be used)

	// initialize random vectors
	for (x=0; x < 4; x++)
	{
		for (y=0; y < 4; y++)
		{
			for (z=0; z < 4; z++)
			{
				randvecs[((z*4 + y)*4 + x)*3 + 0] = (((double)rand() / RAND_MAX) * 2.0 - 1.0) * 0.6;
				randvecs[((z*4 + y)*4 + x)*3 + 1] = (((double)rand() / RAND_MAX) * 2.0 - 1.0) * 0.6;
				randvecs[((z*4 + y)*4 + x)*3 + 2] = (((double)rand() / RAND_MAX) * 2.0 - 1.0) * 0.6;
			}
		}
	}

	// for each face of the cube...
	for (a=0; a < 6; a++)
	{
		// set up the control points for this face
		for (x=0; x < 4; x++)
		{
			for (y=0; y < 4; y++)
			{
				// figure out the right random vector to sample and the right way to interpret it
				switch (a)
				{
					case 0: sampx = 0; sampy = y; sampz = x;    xcoord=2; ycoord=1; zcoord=0; xmult=-1; ymult=1; zmult=1; break;
					case 1: sampx = 3; sampy = y; sampz = 3-x;  xcoord=2; ycoord=1; zcoord=0; xmult=1; ymult=1; zmult=-1; break;
					case 2: sampx = x; sampy = 0; sampz = y;    xcoord=0; ycoord=2; zcoord=1; xmult=1; ymult=-1; zmult=1; break;
					case 3: sampx = x; sampy = 3; sampz = 3-y;  xcoord=0; ycoord=2; zcoord=1; xmult=1; ymult=1; zmult=-1; break;
					case 4: sampx = x; sampy = y; sampz = 3;    xcoord=0; ycoord=1; zcoord=2; xmult=1; ymult=1; zmult=1; break;
					case 5: sampx = 3-x; sampy = y; sampz = 0;  xcoord=0; ycoord=1; zcoord=2; xmult=-1; ymult=1; zmult=-1; break;
				}
				// make a control point
				ctrlpoints[(y*4 + x)*3 + 0] = (x / 3.0) * 2.0 - 1.0   + xmult*randvecs[((sampz*4 + sampy)*4 + sampx)*3 + xcoord];
				ctrlpoints[(y*4 + x)*3 + 1] = (y / 3.0) * 2.0 - 1.0   + ymult*randvecs[((sampz*4 + sampy)*4 + sampx)*3 + ycoord];
				ctrlpoints[(y*4 + x)*3 + 2] = -1.0                    + zmult*randvecs[((sampz*4 + sampy)*4 + sampx)*3 + zcoord];
			}
		}

		glPushMatrix();
			// orient the face properly
			switch (a)
			{
				case 0: glRotated( 90.0, 0.0,1.0,0.0); break;
				case 1: glRotated(-90.0, 0.0,1.0,0.0); break;
				case 2: glRotated(-90.0, 1.0,0.0,0.0); break;
				case 3: glRotated( 90.0, 1.0,0.0,0.0); break;
				case 4: /* no rotation */              break;
				case 5: glRotated(180.0, 0.0,1.0,0.0); break;
			}
			// and render the face
			gluBeginSurface(myNurb);
				gluNurbsSurface(myNurb,
					8,knots, 8,knots,
					4*3, 3,
					&ctrlpoints[0],
					4, 4,
					GL_MAP2_VERTEX_3
				);
			gluEndSurface(myNurb);
		glPopMatrix();
	}
	delete[] ctrlpoints;
	delete[] knots;
	delete[] randvecs;
}

Bar createBar(double weight, int depthsofar, double &height)
{
	Bar bar;
	double leftdist, rightdist;
	double barWeight, barWeightTerm, leftWeight, rightWeight;
	double leftheight, rightheight;

	// randomly choose whether this is an object or not
	bar.isObject = (rand() % 2) == 0;

	// make sure the choice was acceptable
	if (depthsofar > 10 || weight < 0.1)
		bar.isObject = true;
	if (depthsofar < 3)
		bar.isObject = false;

	// set up properties relating to both bars and objects
	bar.rot      = (((double)rand() / RAND_MAX) * 2.0 - 1.0) * 1.0*PI;
	bar.rotvel   = (((double)rand() / RAND_MAX) * 2.0 - 1.0) * PI * 0.2;
	bar.rotaccel =  ((double)rand() / RAND_MAX)              * 0.35 + 0.1;

	bar.stringLength = (((double)rand() / RAND_MAX) * 2.0 + 0.5) / sqrt((double)(depthsofar + 1));

	// set the height so we can figure out how tall the mobile is
	height = bar.stringLength;

	if (!bar.isObject)
	{
		// set up bar-specific data
		leftdist  = bar.data.barData.leftDistance  = -(((double)rand() / RAND_MAX) * 2.5 + 1.0) / (double)(depthsofar + 1);
		rightdist = bar.data.barData.rightDistance =  (((double)rand() / RAND_MAX) * 2.5 + 1.0) / (double)(depthsofar + 1);

		bar.data.barData.left  = new Bar;
		bar.data.barData.right = new Bar;

		// determine how much weight should be handled by each sub-bar
		// i'm using a somewhat complicated formula to determine the weight of each sub-bar in order to keep balance.

		barWeight = barweightperunit * (rightdist - leftdist);
		barWeightTerm = barWeight * (rightdist*rightdist - leftdist*leftdist) / (2.0 * (rightdist - leftdist));

		leftWeight  =  (barWeightTerm + rightdist * (weight - barWeight)) / (rightdist - leftdist);
		rightWeight = -(barWeightTerm +  leftdist * (weight - barWeight)) / (rightdist - leftdist);

		if (leftWeight > 0.0 && rightWeight > 0.0)
		{
			// set up the sub-bars
			*bar.data.barData.left  = createBar(leftWeight , depthsofar + 1, leftheight );
			*bar.data.barData.right = createBar(rightWeight, depthsofar + 1, rightheight);

			if (leftheight > rightheight)
				height += leftheight;
			else
				height += rightheight;
		}
		else
			// changed our minds; we're going to use an object here instead of a bar
			bar.isObject = true;
	}
	if (bar.isObject)
	{
		// set up object-specific data
		bar.data.objectData.weight = weight;

		// create a display list of whatever should be displayed for this object
		// (make a random nurbs surface)
		bar.data.objectData.displaylist = glGenLists(1);
		glNewList(bar.data.objectData.displaylist, GL_COMPILE);
			// choose a random color
			glColor3d(
				((double)rand() / RAND_MAX) * 0.6 + 0.6,
				((double)rand() / RAND_MAX) * 0.6 + 0.6,
				((double)rand() / RAND_MAX) * 0.6 + 0.6
			);

			// make a random nurbs-based object
			renderRandomNurbsObject();
		glEndList();
	}

	return bar;
}

void initTextures()
{
	int a;
	char filename[32];
	int w,h;
	GLubyte *theTexture;
	GLenum format;
	GLint internalformat;

	// set up background textures
	glGenTextures(6, bgtextures);
	for (a=0; a < 6; a++)
	{
		sprintf(filename, "sky%d.png", a);

		// getPNGTextureData reads the PNG file into the theTexture array
		// and returns the format and internalformat which should be used to parse it,
		// along with its width and height.
		if (!getPNGTextureData(filename, theTexture, format,internalformat, w,h)) {
			cerr << "WARNING: Couldn't read " << filename << endl;
			continue;
		}

		glBindTexture(GL_TEXTURE_2D, bgtextures[a]);

		gluBuild2DMipmaps(GL_TEXTURE_2D,    internalformat, w,h,    format, GL_UNSIGNED_BYTE, theTexture);

		delete[] theTexture;
		
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR); // GL_NEAREST_MIPMAP_NEAREST
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR); // GL_LINEAR_MIPMAP_LINEAR

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	}
	skytextureoffset = (1.0 / w) / 2.0 * 1.1; // save edge size of sky textures for later


	// set up wood texture (for bars)
	glGenTextures(1, &bartexture);
	if (!getPNGTextureData("wood.png", theTexture, format,internalformat, w,h)) {
		cerr << "WARNING: Couldn't read wood.png" << endl;
	}
	else {
		glBindTexture(GL_TEXTURE_2D, bartexture);

		gluBuild2DMipmaps(GL_TEXTURE_2D,    internalformat, w,h,    format, GL_UNSIGNED_BYTE, theTexture);

		delete[] theTexture;
		
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	}
}

void init(void)
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	srand(641);

	glShadeModel(GL_SMOOTH);

	myNurb = gluNewNurbsRenderer();
    gluNurbsProperty(myNurb, GLU_SAMPLING_METHOD, GLU_DOMAIN_DISTANCE);
	gluNurbsProperty(myNurb, GLU_U_STEP, 5);
	gluNurbsProperty(myNurb, GLU_V_STEP, 5);
    gluNurbsProperty(myNurb, GLU_DISPLAY_MODE, GLU_FILL);

	glEnable(GL_AUTO_NORMAL);

	initTextures();

	glLightfv(GL_LIGHT0, GL_AMBIENT,  sunambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  sundiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, sunspecular);

	glLightfv(GL_LIGHT1, GL_AMBIENT,  groundlightambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  groundlightdiffuse);
	glLightfv(GL_LIGHT1, GL_SPECULAR, groundlightspecular);

	glLightfv(GL_LIGHT2, GL_AMBIENT,  skylightambient);
	glLightfv(GL_LIGHT2, GL_DIFFUSE,  skylightdiffuse);
	glLightfv(GL_LIGHT2, GL_SPECULAR, skylightspecular);

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fullcolor);

	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	mobilegenerated = false;
	mobileHeight = 0.0;
	viewDist = 1.0;

	starttime = clock();
}

// frees memory
void freeBar(Bar &bar)
{
	if (!bar.isObject)
	{
		freeBar(*bar.data.barData.left);
		delete bar.data.barData.left;

		freeBar(*bar.data.barData.right);
		delete bar.data.barData.right;
	}
	else
	{
		glDeleteLists(bar.data.objectData.displaylist, 1);
	}
}

//=============== RESHAPE =========================

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	GLdouble AspectRatio = (GLdouble)w/(GLdouble)h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if (AspectRatio > 1.0)
	{
		gluPerspective(40.0, AspectRatio, 0.1, 1000.0);
		//gluOrtho2D(-1.0*AspectRatio, 1.0*AspectRatio, -1.0, 1.0);
		xoffset = -1.0*AspectRatio;
		xscale = 2.0*AspectRatio/w;
		yoffset = 1.0;
		yscale = -2.0/h;
	}
	else
	{
		gluPerspective(2.0*atan(tan(40.0/2.0/180.0*PI) / AspectRatio)/PI*180.0, AspectRatio, 0.1, 1000.0);
		//gluOrtho2D(-1.0, 1.0, -1.0/AspectRatio, 1.0/AspectRatio);
		xoffset = -1.0;
		xscale = 2.0/w;
		yoffset = 1.0/AspectRatio;
		yscale = -2.0/AspectRatio/h;
	}
	glMatrixMode(GL_MODELVIEW);
}

void mouse (int button, int state, int x, int y)
{
	//int a;

	switch(button)
	{
		case GLUT_LEFT_BUTTON:
			mousedragging = state == GLUT_DOWN;
			break;

		case GLUT_RIGHT_BUTTON:
			if (state == GLUT_DOWN)
			{	
				srand(clock());

				if (mobilegenerated)
					freeBar(mobile);
				mobile = createBar(mobileweight, 0, mobileHeight);
				viewDist = mobileHeight * 2.0;

				mobilegenerated = true;
			}
			break;
	}
}

void mousemove(int x, int y)
{
	double oldmx, oldmy;

	if (mousedragging)
	{
		oldmx = mx;
		oldmy = my;
	}

	mx = xscale*x + xoffset;
	my = yscale*y + yoffset;

	if (mousedragging)
	{
		viewTheta -= mx-oldmx;
		viewPhi -= my-oldmy;
	}
}

//=============== MAIN =========================

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(50, 50);
	glutCreateWindow(argv[0]);

	init();
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutMouseFunc(mouse);
	glutMotionFunc(mousemove);
	glutPassiveMotionFunc(mousemove);
	glutIdleFunc(animate);
	glutMainLoop();
	return 0;
}
